package me.totalfreedom.totalfreedommod.perms;

import me.totalfreedom.totalfreedommod.rank.DisplayableGroup;
import me.totalfreedom.totalfreedommod.util.FLog;
import net.luckperms.api.model.user.User;
import net.luckperms.api.model.user.UserManager;
import net.luckperms.api.node.NodeType;
import net.luckperms.api.platform.PlayerAdapter;
import org.bukkit.entity.Player;

public class UserWrapper
{
    private final UserManager manager;
    private final PlayerAdapter<Player> playerAdapter;

    public UserWrapper(UserManager manager, PlayerAdapter<Player> adapter)
    {
        this.manager = manager;
        this.playerAdapter = adapter;
    }

    public User getUser(Player player)
    {
        if (!manager.isLoaded(player.getUniqueId()) || !manager.getLoadedUsers().contains(manager.getUser(player.getUniqueId())))
        {
            return manager.loadUser(player.getUniqueId()).join();
        }

        return manager.getUser(player.getUniqueId());
    }

    public void addToGroup(User user, DisplayableGroup group)
    {

        manager.modifyUser(user.getUniqueId(), c ->
                {
                    if (!c.getNodes(NodeType.INHERITANCE)
                            .contains(group.getInheritance()))
                        c.data().add(group.getInheritance());
                })
                .thenRun(() ->
                        FLog.info("Successfully saved user "
                                + user.getUsername()
                                + " to group "
                                + group.getName()));
    }

    public void removeFromGroup(User user, DisplayableGroup group)
    {
        manager.modifyUser(user.getUniqueId(), userConsumer ->
        {
            if (userConsumer.getNodes(NodeType.INHERITANCE)
                    .contains(group.getInheritance()))
                userConsumer.data().remove(group.getInheritance());
        }).thenRun(() ->
                FLog.info("Successfully removed user "
                        + user.getUsername()
                        + " from group "
                        + group.getName()));
    }

    public void saveUser(User user)
    {
        manager.saveUser(user).thenRun(() ->
                FLog.info("Successfully saved user "
                        + user.getUsername()));
    }
}
