package me.totalfreedom.totalfreedommod.rank;

import me.totalfreedom.totalfreedommod.TotalFreedomMod;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.TextColor;
import net.luckperms.api.context.ContextSatisfyMode;
import net.luckperms.api.model.group.Group;
import net.luckperms.api.node.NodeType;
import net.luckperms.api.node.types.InheritanceNode;
import net.luckperms.api.node.types.PermissionNode;
import net.luckperms.api.node.types.PrefixNode;
import net.luckperms.api.node.types.WeightNode;
import org.apache.commons.lang.StringUtils;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class DisplayableGroup implements Displayable
{
    private final Group group;

    private final Component name;

    private final Component abbr;
    private final Component plural;
    private final Component article;
    private final WeightNode weight;

    private final Component tag;

    private final Component coloredTag;

    private final TextColor color;

    private final org.bukkit.ChatColor teamColor;

    private final boolean hasTeam;

    private final boolean hasDefaultLoginMessage;


    private final InheritanceNode inheritance;
    private final PrefixNode prefix;
    private final List<PermissionNode> permissions = new ArrayList<>();


    public DisplayableGroup(String group,
                            Component plural,
                            Component tag,
                            int weight,
                            TextColor color,
                            org.bukkit.ChatColor teamColor,
                            boolean hasTeam,
                            boolean hasDefaultLoginMessage)
    {
        WeightNode tempWeight;
        PrefixNode tempPrefix;
        Group matched = TotalFreedomMod.getPlugin().getHierarchy().getGroup(group); // we don't need to null check because if there is no group this method will also create one.

        this.name = (matched.getDisplayName() != null) ? FUtil.miniMessage(matched.getDisplayName()) : FUtil.miniMessage(matched.getName());
        this.plural = plural;
        this.article = StringUtils.startsWithAny(this.name.toString().toLowerCase(Locale.ROOT), new String[]{"a", "e", "i", "o", "u"}) ? Component.text("an") : Component.text("a");
        this.abbr = tag;
        this.color = color;
        this.teamColor = teamColor;
        this.hasTeam = hasTeam;
        this.coloredTag = tag.color(color);
        this.tag = GroupProvider.OPEN.append(coloredTag).append(GroupProvider.CLOSE);
        this.hasDefaultLoginMessage = hasDefaultLoginMessage;
        tempWeight = WeightNode.builder(weight).build();
        tempPrefix = PrefixNode.builder(FUtil.miniMessage(getTag()), weight).build();

        permissions.addAll(matched.getNodes(NodeType.PERMISSION));

        if (!matched.getNodes(NodeType.PREFIX).contains(tempPrefix)) matched.data().add(tempPrefix);
        else tempPrefix = matched.getNodes(NodeType.PREFIX).iterator().next();

        if (!matched.getNodes(NodeType.WEIGHT).contains(tempWeight)) matched.data().add(tempWeight);
        else tempWeight = matched.getNodes(NodeType.WEIGHT).iterator().next();

        this.prefix = tempPrefix;
        this.weight = tempWeight;
        this.group = matched;
        this.inheritance = InheritanceNode.builder(this.group).build();
        this.save();
    }

    /**
     * Checks the weight of this rank instance against the given rank.
     *
     * @param rank The rank to check against.
     * @return True if the given rank is less than or equal to the weight of this specific rank instance.
     */
    public boolean weightCheckAgainst(@NotNull DisplayableGroup rank)
    {
        return rank.getWeight().getWeight() <= getWeight().getWeight();
    }

    public void addPermission(String permission)
    {
        PermissionNode node = PermissionNode.builder(permission).build();
        for (PermissionNode it : group.getNodes(NodeType.PERMISSION))
        {
            if (it.getContexts().isSatisfiedBy(node.getContexts())
                    || it.getKey().equals(node.getKey()))
                continue;

            group.data().add(node);
            permissions.add(node);
            save();
        }
    }

    public void removePermission(String permission)
    {
        PermissionNode node = PermissionNode.builder(permission).build();
        for (PermissionNode it : group.getNodes(NodeType.PERMISSION))
        {
            if (it.getContexts().isSatisfiedBy(node.getContexts(), ContextSatisfyMode.AT_LEAST_ONE_VALUE_PER_KEY)
                    || it.getKey().equals(node.getKey()))
            {
                group.data().remove(node);
                permissions.remove(node);
                save();
            }
        }
    }

    public WeightNode getWeight()
    {
        return weight;
    }

    public Group getLuckPermsGroup()
    {
        return this.group;
    }

    @Override
    public Component getArticle()
    {
        return this.article;
    }

    @Override
    public Component getName()
    {
        return name;
    }

    @Override
    public Component getTag()
    {
        return tag;
    }

    @Override
    public Component getAbbr()
    {
        return abbr;
    }

    @Override
    public Component getPlural()
    {
        return plural;
    }

    @Override
    public TextColor getColor()
    {
        return color;
    }

    @Override
    public org.bukkit.ChatColor getTeamColor()
    {
        return teamColor;
    }

    @Override
    public Component getColoredName()
    {
        return name.color(color);
    }

    @Override
    public Component getColoredTag()
    {
        return coloredTag;
    }

    @Override
    public Component getColoredLoginMessage()
    {
        return article.append(Component.text(" ")).append(name.color(color));
    }

    @Override
    public boolean hasTeam()
    {
        return hasTeam;
    }

    @Override
    public boolean hasDefaultLoginMessage()
    {
        return hasDefaultLoginMessage;
    }

    public InheritanceNode getInheritance()
    {
        return inheritance;
    }

    public PrefixNode getPrefix()
    {
        return prefix;
    }

    public List<PermissionNode> getPermissions()
    {
        return permissions;
    }

    public void save()
    {
        TotalFreedomMod.getPlugin().getHierarchy().gw().saveGroup(group.getName());
    }
}
